/* -*- Mode: c++ -*- */
/*
 * Copyright 2001 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/*
 *  Copyright 1997 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


#ifndef _VRFakeGuppiSOURCE_H_
#define _VRFakeGuppiSOURCE_H_

extern "C" {
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
}

#include <VrSource.h>

template<class oType> 
class VrFakeGuppiSource: public VrSource<oType> {
protected:
  int bufferSize;
  int gupfd;
public: 
  virtual const char *name() { return "VrFakeGuppiSource"; }

  virtual float memoryTouched() {
    return 0; //no outputs are cached
  }
  virtual int work2(VrSampleRange output, void *o[]);
  VrFakeGuppiSource(double sampling_freq);
};

template<class oType> int
VrFakeGuppiSource<oType>::work2(VrSampleRange output, void *o[])
{
  sync(output.index);
  return output.size;
}

template<class oType>
VrFakeGuppiSource<oType>::VrFakeGuppiSource(double sampling_freq)
  :gupfd(-1)
{
  // add a query here to find out what data type(s) the FakeGuppi supports
  // Defaults to char for now
  bufferSize = 20;
  setOutputSize (bufferSize*getpagesize()/sizeof(oType));
} 
#endif 
